/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui;

import gui.tray.*;
import map.*;
import util.*;
import controller.*;
import agent.*;
import agent.autonomous.*;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;



/* ===================================================================== */
/** This class is an extension of a JPanel.  It's backgound is that of
 *  scale map corresponding to real world GPS coordinates.  Agents are drawn
 *  on the JPanel, and can be dragged around the map. also wirelss ranges
 *  are drawn on each Agent if requested by the iSIM user.
 *
 */
public class ChartComp extends JPanel implements MouseListener, MouseMotionListener{


	private SimulatorFrame parent;

	/** The controller of the simulation. */
	private Controller controller;

	/** The chart data that contains map display and gps information. */
  	private ChartData chartdata;

	/** Holds the map background image. */
	private Image img;

	/** The point of key/center agent for drawing. */
	private Point pos;

	/** Offset position on an agent when mouse-dragging. */
	private int x_offset = 0;

	/** Offset position on an agent when mouse-dragging. */
	private int y_offset = 0;

	/** A time for keeping track of double-clicking. */
	private Date doubleclick = new Date();

	/** A Dialog Window for display User information. */
	private ObjectDialog objectDialog;

	/** A Dialog Window for display User information. */
	private RoomDialog roomDialog;

	/** A Dialog Window for display User information. */
	private PropDialog propDialog;

	private Building building;

	private Floor floor;


	/** True if this JComponent is to display wireless ranges. */
	private boolean ranges;

	/** True if when user clicks to move an agent on map. */
	private boolean agent = false;

	private Object current;
	private Object clicked;

	private Agent drag;


	private Point ref;

	private double[] oHead = new double[25];


    /* ===================================================================== */
    /** The constructor sets up mouse listeners and sets up its own variables
     *  with the given ones.
     *
     *  @param  controller   The controller of simulation.
     *  @param  ranges   True if displaying wireless ranges.
     *
     */
  	public ChartComp(SimulatorFrame p, Building b, Controller c, boolean r){

		parent = p;
		building = b;
		controller = c;
		ranges = r;
		loadChart("background.xml");

		// Sign up listeners
		this.addMouseListener(this);
		this.addMouseMotionListener(this);




  	} //Constructor





    /* ===================================================================== */
    /** This method returns our preferred size which should be the size of the
     *  map image that this object displays.
     *
     *  @return  Dimension   The size of this JPanel with map image.
     *
     */
  	synchronized public Dimension getPreferredSize(){

    	if (img!=null){

			// Wait until the image is loaded so we know the size
			int i=0;
			while ((i<30) && (img.getWidth(this)<0)){

				try {this.wait(100);} catch (InterruptedException e) {}
				i++;
			}


			return new Dimension(img.getWidth(this), img.getHeight(this));

    	}else{

      		return new Dimension(100,100);
    	}

  	} //method: getPreferredSize



    /* ===================================================================== */
    /** This method is used whenever this JPanel gets painted.  The method
     *  draws all agents onto the map, and draws their wireless ranges if
     *  wanted
     *
     *  @param  g   The Graphics object for drawing images and circles.
     *  @param  users   The user agents to draw.
     *  @param  points   The access point agents to draw.
     *
     */
  	synchronized void draw(Graphics g, Vector users, Vector points){

		//Draw the floor diagram.
		// Make sure there is something to draw
   		if( floor != null ){


			// Draw map information
			try{

				Vector rooms = floor.getRooms();

				for(int i=0; i<rooms.size(); i++ ){

					Room rm = (Room)rooms.elementAt(i);
					Vector walls = rm.getWalls();
					Vector furns = rm.getFurnishings();

					for(int j=0; j<walls.size(); j++ ){

						Wall wall = (Wall)walls.elementAt(j);
						Position p1 = wall.getPosition1();
						Position p2 = wall.getPosition2();

						Point pos1 = chartdata.transPos(p1);
						Point pos2 = chartdata.transPos(p2);

						if( rm.equals(current) || wall.equals(current) || floor.equals(current) )
							g.setColor(Color.black );
						else
							g.setColor(Color.lightGray );
						g.drawLine( pos1.x, pos1.y, pos2.x, pos2.y );
						g.drawLine( pos1.x+1, pos1.y, pos2.x+1, pos2.y );
						g.drawLine( pos1.x, pos1.y+1, pos2.x, pos2.y+1 );

						Vector openings = wall.getOpenings();

						for(int k=0; k<openings.size(); k++ ){

							Opening opening = (Opening)openings.elementAt(k);
							Position op1 = opening.getPosition1();
							Position op2 = opening.getPosition2();

							Point opos1 = chartdata.transPos(op1);
							Point opos2 = chartdata.transPos(op2);

							if( opening.equals(current) || rm.equals(current) || wall.equals(current) || floor.equals(current) ){
								if( opening.isOpen() )
									g.setColor(Color.blue );
								else
									g.setColor(Color.red );
							}else{
								g.setColor(new Color(150,150,250) );
							}
							g.drawLine( opos1.x, opos1.y, opos2.x, opos2.y );
							g.drawLine( opos1.x+1, opos1.y, opos2.x+1, opos2.y );
							g.drawLine( opos1.x, opos1.y+1, opos2.x, opos2.y+1 );
							g.drawLine( opos1.x+2, opos1.y, opos2.x+2, opos2.y );
							g.drawLine( opos1.x, opos1.y+2, opos2.x, opos2.y+2 );
							g.drawLine( opos1.x-1, opos1.y, opos2.x-1, opos2.y );
							g.drawLine( opos1.x, opos1.y-1, opos2.x, opos2.y-1 );


						} //for


					} //for

					for(int j=0; j<furns.size(); j++ ){

						Furnishing furn = (Furnishing)furns.elementAt(j);
						Position p1 = furn.getPosition1();
						Position p2 = furn.getPosition2();

						Point pos1 = chartdata.transPos(p1);
						Point pos2 = chartdata.transPos(p2);

						//System.out.println("   " +pos1.x+" "+ pos1.y+" "+ (pos2.x-pos1.x)+" "+ (pos2.y-pos1.y));

						if( furn.equals(current) || rm.equals(current) || floor.equals(current) )
							g.setColor(Color.black );
						else
							g.setColor(Color.lightGray );

						int width, height, x, y;
						if( pos1.x < pos2.x ){
							x = pos1.x;
							width = pos2.x - pos1.x;
						}else{
							x = pos2.x;
							width = pos1.x - pos2.x;
						}
						if( pos1.y < pos2.y ){
							y = pos1.y;
							height = pos2.y - pos1.y;
						}else{
							y = pos2.y;
							height = pos1.y - pos2.y;
						}

						g.drawRect( x, y, width, height );
						g.drawRect( x+1, y+1, width, height );


					} //for


				} //for

			}catch(Exception ee){}


			// Draw a circle
			if( clicked != null ){

				g.setColor(Color.green );
				Position p1;
				Position p2;
				if(clicked instanceof Wall){
					Wall wall = (Wall)clicked;
					p1 = wall.getPosition1();
					p2 = wall.getPosition2();
				}else if(clicked instanceof Opening){
					Opening wall = (Opening)clicked;
					p1 = wall.getPosition1();
					p2 = wall.getPosition2();
				}else{
					Furnishing wall = (Furnishing)clicked;
					p1 = wall.getPosition1();
					p2 = wall.getPosition2();
				}

				Point pos1 = chartdata.transPos(p1);
				Point pos2 = chartdata.transPos(p2);

				g.drawOval(pos1.x-8,pos1.y-7,16,16);
				g.drawLine(pos1.x-4,pos1.y,pos1.x-12,pos1.y);
				g.drawLine(pos1.x+4,pos1.y,pos1.x+12,pos1.y);
				g.drawLine(pos1.x,pos1.y-4,pos1.x,pos1.y-12);
				g.drawLine(pos1.x,pos1.y+4,pos1.x,pos1.y+12);

				g.drawOval(pos2.x-8,pos2.y-7,16,16);
				g.drawLine(pos2.x-4,pos2.y,pos2.x-12,pos2.y);
				g.drawLine(pos2.x+4,pos2.y,pos2.x+12,pos2.y);
				g.drawLine(pos2.x,pos2.y-4,pos2.x,pos2.y-12);
				g.drawLine(pos2.x,pos2.y+4,pos2.x,pos2.y+12);

			}

		} //if

	//Draw the agents
		// Make sure there is an agent in the sim

   		if( pos!=null ){

			Point t1;
			Point t2;
			Image img2;

			// Draw users
			for(int i=0; i<users.size(); i++ ){

				if( ((User)users.elementAt(i)).getFloor() == floor.getNumber() ){
					pos = ((User)users.elementAt(i)).getLocation();

					t1 = ((User)users.elementAt(i)).getLocation();
					t2 = controller.getCurrentAgent().getLocation();

					if( !(((User)users.elementAt(i)) instanceof Fire) ){
						if( ((User)users.elementAt(i)).isDead() ){
							img2 = getToolkit().getImage("files/agents/dead.gif");
						}else{
							int hding = (int)(((User)users.elementAt(i)).getPosition().getHeading());
							int part = hding/45;
							int rem = hding%45;
							int deg = part*45;
							if(rem>=(45/2))
								deg += 45;
							if(deg == 360)
								deg = 0;
							img2 = getToolkit().getImage("files/agents/"+((User)users.elementAt(i)).getName()+"_"+deg+".gif");
						}
						//draw agent name/port
						g.setColor(Color.black);
						g.setFont(new Font("Default", Font.PLAIN, 10));
						g.drawString(""+((User)users.elementAt(i)).getPort(), pos.x+4,pos.y+42);
					}else{
						img2 = getToolkit().getImage("files/agents/fire.gif");
					}
					g.drawImage(img2, pos.x, pos.y, this);

					// Draw wireless range circles for the user.
					if(ranges || ((User)users.elementAt(i)) instanceof Fire){
						int circles = (int)((((User)users.elementAt(i)).getRange())/10);
						int ind = 0;
						for(int j=10; j<=((User)users.elementAt(i)).getRange(); j+=10){
							Color c = Color.blue;
							if( ((User)users.elementAt(i)) instanceof Fire ){
								c = Color.red;
							}
							g.setColor(new Color(c.getRed(), c.getGreen(), c.getBlue(), 255-ind*(240/circles) ));
							g.drawOval(pos.x-j+15,pos.y-j+15,j*2,j*2);
							ind++;
						}
					}
				}

			}//for


			// Draw access points
			for(int i=0; i< points.size(); i++ ){

				if( ((Agent)points.elementAt(i)).getFloor() == floor.getNumber() ){
				pos = ((Agent)points.elementAt(i)).getLocation();

				t1 = ((Agent)points.elementAt(i)).getLocation();
				t2 = controller.getCurrentAgent().getLocation();

				img2 = getToolkit().getImage("files/agents/accesspoint.gif");
				g.drawImage(img2, pos.x, pos.y, this);

				// Draw wireless range circles for the access point.
				if(ranges){
					int circles = (int)((((Agent)points.elementAt(i)).getRange())/10);
					int ind = 0;
					for(int j=10; j<=((Agent)points.elementAt(i)).getRange(); j+=10){
						Color c = Color.blue;
						g.setColor(new Color(c.getRed(), c.getGreen(), c.getBlue(), 255-ind*(240/circles) ));
						g.drawOval(pos.x-j+8,pos.y-j+6,j*2,j*2);
						ind++;
					}
				}
				}


			} //for

    	} //if



  	} //method: draw






    /* ===================================================================== */
    /** This method is used to make the JPanel re draw itself, usually because
     *  some information or location has caused an Agent to change or created
     *  a new Agent in the Simulator.
     *
     *  @param  pos   The center point, usually point of current agent.
     *  @param  users   The user agents to draw.
     *  @param  points   The access point agents to draw.
     *
     */
  	synchronized public void updatePos(Point pos, Vector users, Vector points){

		// Get grphics to draw
		Graphics g=getGraphics();

		// Make old drawing
		draw(g, users, points);

		// Set the point, it may be null, or real
		this.pos=pos;

		// Now redraw.
		draw(g, users, points);

		g.dispose();

  	} //method: updatePos





    /* ===================================================================== */
    /** This method is used in case of some time out event occurs to re draw
     *  this JPanel.
     *
     */
  	synchronized public void timeOut(){

    	Graphics g=getGraphics();
    	draw(g, controller.getUsers(), controller.getPoints());

    	draw(g, controller.getUsers(), controller.getPoints());
    	g.dispose();

  	} //method: timeOut



    /* ===================================================================== */
    /** This method is used by JPanels for updating its display, it is safe
     *  to just call paint in this JPanel.
     *
     *  @param  g   The Graphics object for drawing images and circles.
     *
     */
  	public void update(Graphics g){

    	paint(g);

  	} //method: update




    /* ===================================================================== */
    /** This method will get the Graphics object for painting this JPanel, and
     *  then call on paint(Graphics g).
     *
     *  @param  g   The Graphics object for drawing images and circles.
     *
     */
  	synchronized public void paint(){

    	Graphics g=getGraphics();
    	paint(g);
    	g.dispose();

  	} //method: paint



    /* ===================================================================== */
    /** This method is used to paint this JPanel, it will first make sure the
     *  map image is set, and then will call the draw() method to paint all
     *  agents and wireless ranges.
     *
     *  @param  g   The Graphics object for drawing images and circles.
     *
     */
  	synchronized public void paint(Graphics g){


		if (img!=null) {
			g.drawImage(img, 0, 0, this);
		}


		g.setPaintMode();
		g.setColor(Color.blue);

		draw(g,controller.getUsers(), controller.getPoints());
		g.dispose();

	} //method: paint






    /* ===================================================================== */
    /** This method checks for two kinds of mouse clicks.  If first checks for
     *  a right mouse click which is for rotating a user.(changing user's
     *  facing direction)
     *  This mehtod also checks for mouse left button double clicks.  Based on
     *  the location of click and the timing, it will lauch a PropDialog for
     *  the User that was double clicked on.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseClicked(MouseEvent e){

		int btn = e.getButton();
		Point p = e.getPoint();


		// Check for right click.
		if( btn == 3 ){

			// try to carry a person.
			if( heal() )
				controller.fireControllerChange();

			// try to heal a person
			else if( carry() )
				controller.fireControllerChange();

			// try to water a fire.
			else if( water() )
				controller.fireControllerChange();

			// try to open a door.
			else if( open(p) )
				controller.fireControllerChange();

		// Check for left click.
		}else if( btn == 1 ){

			double d = 100000;
			double temp;

			// Make sure we clicked on a User or AccessPoint.
			for(int i=0; i<controller.getUsers().size(); i++){

				temp = diff(p, ((User)controller.getUsers().elementAt(i)).getLocation());

				if( temp < d ){
					d = temp;
					controller.setCurrentAgent( ((User)controller.getUsers().elementAt(i)) );
				}

			}
			for(int i=0; i<controller.getPoints().size(); i++){

				temp = diff(p, ((Agent)controller.getPoints().elementAt(i)).getLocation());

				if( temp < d ){
					d = temp;
					controller.setCurrentAgent( ((Agent)controller.getPoints().elementAt(i)) );
				}

			}

			// Now check to see if closest Agent is User, and we are clicking on User.
			if( controller.getCurrentAgent() != null && controller.getCurrentAgent() instanceof User ){
				if( x_offset <= 32 && x_offset >= 0 && y_offset <= 32 && y_offset >= 0){


					// Check time to see if it is double-click. double-click if < .75 sec.
					if( ((new Date()).getTime() - doubleclick.getTime()) < 750 ){

						// Close old PropDialog if it exists.
						try{
							propDialog.dispose();
						}catch(Exception exc){}

						// Open a PropDialog
						propDialog = new PropDialog( controller, controller.getFrame() );
						controller.fireControllerChange();
						return;

					}
				}

			} //if


			// Look at map stuff now.
			p = new Point( (int)(Math.round((p.getX()-2)/10)*10), (int)(Math.round((p.getY()-2)/10)*10) );

			clicked = null;


			// Draw map information
			try{

				Vector rooms = floor.getRooms();

				for(int i=0; i<rooms.size(); i++ ){

					Room rm = (Room)rooms.elementAt(i);
					Vector walls = rm.getWalls();
					Vector furns = rm.getFurnishings();

					for(int j=0; j<walls.size(); j++ ){

						Wall wall = (Wall)walls.elementAt(j);
						Position p1 = wall.getPosition1();
						Position p2 = wall.getPosition2();

						Point pos1 = chartdata.transPos(p1);
						Point pos2 = chartdata.transPos(p2);

						//check if a wall
						double s = 0;
						double s1 = 0;
						double s2 = 0;
						try{
							s = (pos1.getY()-pos2.getY())/(pos1.getX()-pos2.getX());
						}catch(Exception ee){}
						try{
							s1 = (pos1.getY()-p.getY())/(pos1.getX()-p.getX());
						}catch(Exception ee){}
						try{
							s2 = (p.getY()-pos2.getY())/(p.getX()-pos2.getX());
						}catch(Exception ee){}

						s = Math.abs(s);
						s1 = Math.abs(s1);
						s2 = Math.abs(s2);

						if( s > 1000000 ) s = 0;

						double d1 = s-s1;
						double d2 = s-s2;

						if( s > 2 ){
							try{
								d1 = d1/s;
							}catch(Exception ee){}
							try{
								d2 = d2/s;
							}catch(Exception ee){}
						}

						if( s1 > 1000000 || new Double(s1).isNaN() ) d1 = 0;
						if( s2 > 1000000 || new Double(s2).isNaN() ) d2 = 0;

						if( d1 < 0)
							d1 = d1 *-1.0;
						if( d2 < 0)
							d2 = d2 *-1.0;

						if( d1 < .2 && d2 < .2 && isWall(p, wall) ){
							clicked = wall;
						}


						Vector openings = wall.getOpenings();

						for(int k=0; k<openings.size(); k++ ){

							Opening opening = (Opening)openings.elementAt(k);
							Position op1 = opening.getPosition1();
							Position op2 = opening.getPosition2();

							Point opos1 = chartdata.transPos(op1);
							Point opos2 = chartdata.transPos(op2);

							//check if opening
							s = 0;
							s1 = 0;
							s2 = 0;
							try{
								s = (opos1.getY()-opos2.getY())/(opos1.getX()-opos2.getX());
							}catch(Exception ee){}
							try{
								s1 = (opos1.getY()-p.getY())/(opos1.getX()-p.getX());
							}catch(Exception ee){}
							try{
								s2 = (p.getY()-opos2.getY())/(p.getX()-opos2.getX());
							}catch(Exception ee){}

							//System.out.println(" Slope: "+s );
							//System.out.println("      Slopes: "+s1+"   "+s2 );
							//infinity check
							s = Math.abs(s);
							s1 = Math.abs(s1);
							s2 = Math.abs(s2);

							if( s > 1000000 ) s = 0;

							d1 = s-s1;
							d2 = s-s2;

							if( s > 2 ){
								try{
									d1 = d1/s;
								}catch(Exception ee){}
								try{
									d2 = d2/s;
								}catch(Exception ee){}
							}

							if( s1 > 1000000 || new Double(s1).isNaN() ) d1 = 0;
							if( s2 > 1000000 || new Double(s2).isNaN() ) d2 = 0;

							if( d1 < 0)
								d1 = d1 *-1.0;
							if( d2 < 0)
								d2 = d2 *-1.0;

							if( d1 < .2 && d2 < .2 && isOpening(p, opening) ){
								clicked = opening;
							}


						} //for


					} //for

					for(int j=0; j<furns.size(); j++ ){

						Furnishing furn = (Furnishing)furns.elementAt(j);
						Position p1 = furn.getPosition1();
						Position p2 = furn.getPosition2();

						Point pos1 = chartdata.transPos(p1);
						Point pos2 = chartdata.transPos(p2);

						if( p.x >= pos1.x && p.x <= pos2.x && p.y >= pos1.y && p.y <= pos2.y ){
							clicked = furn;
						}else if( p.x <= pos1.x && p.x >= pos2.x && p.y <= pos1.y && p.y >= pos2.y ){
							clicked = furn;
						}else if( p.x >= pos1.x && p.x <= pos2.x && p.y <= pos1.y && p.y >= pos2.y ){
							clicked = furn;
						}else if( p.x <= pos1.x && p.x >= pos2.x && p.y >= pos1.y && p.y <= pos2.y ){
							clicked = furn;
						}


					} //for

				} //for

			}catch(Exception ee){}


			if( clicked != null ){
				// Check time to see if it is double-click. double-click if < .75 sec.
				if( ((new Date()).getTime() - doubleclick.getTime()) < 750 ){

					// Close old PropDialog if it exists.
					try{
						objectDialog.dispose();
					}catch(Exception exc){}

					// Open a PropDialog
					objectDialog = new ObjectDialog( clicked, parent );

				}else{
					//Get time of click for checking for double-click next time.
					doubleclick = new Date();
				}
			}else if( isRoom(p) != null ){

				// Check time to see if it is double-click. double-click if < .75 sec.
				if( ((new Date()).getTime() - doubleclick.getTime()) < 750 ){

					// Close old PropDialog if it exists.
					try{
						roomDialog.dispose();
					}catch(Exception exc){}

					// Open a PropDialog
					roomDialog = new RoomDialog( isRoom(p), parent );

				}else{
					//Get time of click for checking for double-click next time.
					doubleclick = new Date();
				}


			}else{
				doubleclick = new Date();
			}



		}
		this.repaint();


	} //method: mouseClicked





    /* ===================================================================== */
    /** This method is used to get the offset position when we click and drag
     *  an agent on the map.  Offset is needed for correctly drawing the agent
     *  on the map.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mousePressed(MouseEvent e){

		int btn = e.getButton();
		Point p = e.getPoint();
		ref = p;

		// See if we are clicking and dragging with left button.
		if( btn == 1 )
			agent = true;


		double d = 100000;
		double temp;

		// Find the closest User or AccessPoint.
		for(int i=0; i<controller.getUsers().size(); i++){

			temp = diff(p, ((User)controller.getUsers().elementAt(i)).getLocation());

			if( temp < d ){
				d = temp;
				controller.setCurrentAgent( ((User)controller.getUsers().elementAt(i)) );
			}

		}
		for(int i=0; i<controller.getPoints().size(); i++){

			temp = diff(p, ((Agent)controller.getPoints().elementAt(i)).getLocation());

			if( temp < d ){
				d = temp;
				controller.setCurrentAgent( ((Agent)controller.getPoints().elementAt(i)) );
			}

		}

		// Calulate the offset from agent location and the click point.
		if( controller.getCurrentAgent() != null ){
			Point a = controller.getCurrentAgent().getLocation();

			x_offset = (int)(p.getX() - a.getX());
			y_offset = (int)(p.getY() - a.getY());

		}

		// Make sure the closest Agent is VERY close to the drag point.
		if( controller.getCurrentAgent() != null ){
			if( x_offset <= 32 && x_offset >= 0 && y_offset <= 32 && y_offset >= 0){
				drag =  controller.getCurrentAgent();
			}
		}



	} //method: mousePressed




    /* ===================================================================== */
    /** This method lets the object know that the user has released from a
     *  mouse drag.  It sends a location change event, so that the location
     *  of the current user can be sent to the iPAQ application.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseReleased(MouseEvent e){

		int btn = e.getButton();
		Point p = e.getPoint();

		drag = null;

		if( btn == 1 )
			agent = false;

		// Make sure there is an agent, set new GPS location, and send
		// the change notification.
		if( controller.getCurrentAgent() != null ){

			Position old = controller.getCurrentAgent().getPosition();
			Position chart_pos = chartdata.transPos(p);


			chart_pos.setHeading( old.getHeading() );
			chart_pos.setTime( old.getTime() );

			controller.getCurrentAgent().setPosition( chart_pos );

			if( controller.getCurrentAgent() instanceof User ){
				controller.locationChange(((User)(controller.getCurrentAgent())));
			}

			this.setCursor( Cursor.getDefaultCursor() );


			controller.fireControllerChange();

		} //if


	} //method: mouseReleased




    /* ===================================================================== */
    /** This method is for moving an Agent around on the map.  It determines
     *  the closest Agent to the drag point, determines if that point is on
     *  an Agent, and then redraws the Agent to the new location based on
     *  offsets gotten from mousePressed method.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseDragged(MouseEvent e){

		Point p = e.getPoint();
		int btn = e.getButton();


		if( btn == 0 ){

			// Make sure the closest Agent is VERY close to the drag point.
			if( controller.getCurrentAgent() != null && !(controller.getCurrentAgent().isDead()) ){
				if( drag != null ){

				//inforce walls here!!!!!
					// Do not allow dragging to off the map.
					if( (int)p.getX()-x_offset > -15 && (int)p.getY()-y_offset > -15
					   && (int)p.getX()-x_offset < (this.getSize().getWidth()-20)
					   && (int)p.getY()-y_offset < (this.getSize().getHeight()-20) ){

						// Get new location and repaint.
						p = new Point((int)p.getX()-x_offset, (int)p.getY()-y_offset);
						Point pold = controller.getCurrentAgent().getLocation();

						//Calculate the heading and round by 45
						//int deg = (int)(Math.round(getDegrees(ref, p)/45.0) *45);
						double deg = getDegrees(ref,p);
						for(int i=24;i>0;i--)
							oHead[i] = oHead[i-1];
						oHead[0] = deg;
						double sum = 0.0;
						for(int i=0;i<25;i++)
							sum += oHead[i];
						deg = sum/25.0;
						//make from 0 to 360 values.
						if( ref.x < p.x ){
							if( deg < 0 )
								deg += 360;
						}else{
							deg = 180 - deg;
						}
						ref = pold;

						if( controller.getCurrentAgent() instanceof Fire ){
							controller.getCurrentAgent().setLocation(p);
						}else if( canMove(p,controller.getCurrentAgent().getLocation()) ){

							if( controller.getCurrentAgent().getCarrying() != null ){
								int x = controller.getCurrentAgent().getLocation().x - p.x;
								int y = controller.getCurrentAgent().getLocation().y - p.y;
								Point old = controller.getCurrentAgent().getCarrying().getLocation();
								controller.getCurrentAgent().getCarrying().setLocation(new Point(old.x-x,old.y-y) );
								controller.getCurrentAgent().getCarrying().getPosition().setHeading( deg );
							}
							controller.getCurrentAgent().setLocation(p);
							controller.getCurrentAgent().getPosition().setHeading( deg );
						}
						repaint();
					}
				}

			} //if

		} //if( btn == 0 )


	} //method: mouseDragged



    /* ===================================================================== */
    /** This method
     *
     *  @param  e
     *
     */
 	public void mouseMoved(MouseEvent e){

		Point p = e.getPoint();
		Point p1 = new Point((int)(Math.round((p.getX()-2)/10)*10), (int)(Math.round((p.getY()-2)/10)*10));

		Position chart_pos = chartdata.transPos(p1);

		parent.setCoordinate("Lat: "+ (int)(chart_pos.getLatitude()/100) +"."+ (int)(((chart_pos.getLatitude()/100)-(int)(chart_pos.getLatitude()/100))/60*100000000) + ",   Lng: "+(int)(chart_pos.getLongitude()/100) +"."+ (int)(((chart_pos.getLongitude()/100)-(int)(chart_pos.getLongitude()/100))/60*-100000000) );

		//repaint();

	} //method: mouseMoved


    /* ===================================================================== */
    /** This method not used.
     *
     *  @param  e
     *
     */
	public void mouseEntered(MouseEvent e){} //method: mouseEntered
    /* ===================================================================== */
    /** This method not used.
     *
     *  @param  e
     *
     */
	public void mouseExited(MouseEvent e){} //method: mouseExited






	public Building getBuilding(){

		return building;

	}


	public void setBuilding(Building b){

		building = b;

	}


	public SimulatorFrame getMapFrame(){

		return parent;

	}


	public void setFloor(Floor f){

		floor = f;
	}


	public Floor getFloor(){

		return floor;
	}


	public void setCurrent(Object o){
		current = o;
	}

	//try to carry an unconscious person
	public boolean carry(){

		Vector users = controller.getUsers();
		Agent a = controller.getCurrentAgent();

		if( !(a.isDead()) ){

			for(int i=0; i<users.size(); i++ ){

				if( ((User)users.elementAt(i)).isDead() ){
					if( diff( ((User)users.elementAt(i)).getLocation(), a.getLocation() ) < 32 ){
						if( a.getCarrying() == null )
							a.setCarrying( ((User)users.elementAt(i)) );
						else
							a.setCarrying(null);

						return true;
					}
				}

			}
		}
		return false;


	}

	//try to water a fire
	public boolean water(){

		Vector users = controller.getUsers();
		Agent a = controller.getCurrentAgent();

		if( !(a.isDead()) ){

			for(int i=0; i<users.size(); i++ ){

				if( ((User)users.elementAt(i)) instanceof Fire ){
					if( diff( ((User)users.elementAt(i)).getLocation(), a.getLocation() ) < 50 ){

						int health = ((User)users.elementAt(i)).getHealth();
						health = (int)Math.round( health - 10 );
						((User)users.elementAt(i)).setHealth(health);


						//kill if health is zero
						if( health <= 0 ){
							controller.removeUser((User)users.elementAt(i));
						}
						return true;
					}
				}

			}
		}
		return false;


	}

	//try to open/close a door
	public boolean open(Point p){

		Vector users = controller.getUsers();
		Agent a = controller.getCurrentAgent();

		if( !(a.isDead()) ){

			Opening opening = nearOpening(p, new Point( p.x, p.y+25) );
			Opening opening2 = nearOpening2(p, new Point( p.x, p.y+25) );
			if( opening != null ){
				//System.out.println("   we should be opening closing");
				if( opening.isOpen() ){
					opening.setOpen(false);
					try{opening2.setOpen(false);}catch(Exception e){}
				}else{
					opening.setOpen(true);
					try{opening2.setOpen(true);}catch(Exception e){}
				}
				return true;
			}

			opening = nearOpening(p, new Point( p.x, p.y-25) );
			opening2 = nearOpening2(p, new Point( p.x, p.y-25) );
			if( opening != null ){
				//System.out.println("   we should be opening closing");
				if( opening.isOpen() ){
					opening.setOpen(false);
					try{opening2.setOpen(false);}catch(Exception e){}
				}else{
					opening.setOpen(true);
					try{opening2.setOpen(true);}catch(Exception e){}
				}
				return true;
			}
			opening = nearOpening(p, new Point( p.x-25, p.y) );
			opening2 = nearOpening2(p, new Point( p.x-25, p.y) );
			if( opening != null ){
				//System.out.println("   we should be opening closing");
				if( opening.isOpen() ){
					opening.setOpen(false);
					try{opening2.setOpen(false);}catch(Exception e){}
				}else{
					opening.setOpen(true);
					try{opening2.setOpen(true);}catch(Exception e){}
				}
				return true;
			}
			opening = nearOpening(p, new Point( p.x+25, p.y) );
			opening2 = nearOpening2(p, new Point( p.x+25, p.y) );
			if( opening != null ){
				//System.out.println("   we should be opening closing");
				if( opening.isOpen() ){
					opening.setOpen(false);
					try{opening2.setOpen(false);}catch(Exception e){}
				}else{
					opening.setOpen(true);
					try{opening2.setOpen(true);}catch(Exception e){}
				}
				return true;
			}


		}
		return false;


	}


	public boolean heal(){

		Vector users = controller.getUsers();
		Agent g = controller.getCurrentAgent();

		if( !(g.isDead()) && (g instanceof User) ){
			User a = (User)g;
			if( a.getCarrying() != null ){
				a.setCarrying(null);
				return true;
			}else{

				for(int i=0; i<users.size(); i++ ){
					User u = ((User)users.elementAt(i));
					if( u.getPort() != a.getPort() && !(u instanceof Fire) ){
						if( diff( u.getLocation(), a.getLocation() ) < 32 ){
							Room room = isRoom(a.getLocation());
							if( room == null || room.getTemperature() <= 100 && room.getOxygen() >= 1.0 ){
								if( u.getHealth() <= 95 )
									u.setHealth(u.getHealth()+5);
								if( u.getOxygen() <= 95 )
									u.setOxygen(u.getOxygen()+5);
								if( u.getHealth() > 0 )
									u.setDead(false);
								return true;
							}
						}
					}

				}
			}
		}
		return false;

	}




	//Determines if a wall, opening, or furnishing blocks the way
	public Opening nearOpening(Point to, Point from){


		if( floor != null ){

			Vector rooms = floor.getRooms();

			for(int i=0; i<rooms.size(); i++ ){

				Room rm = (Room)rooms.elementAt(i);
				Vector walls = rm.getWalls();

				for(int j=0; j<walls.size(); j++ ){

					Wall wall = (Wall)walls.elementAt(j);

					Vector openings = wall.getOpenings();

					for(int k=0; k<openings.size(); k++ ){

						Opening opening = (Opening)openings.elementAt(k);
						Position op1 = opening.getPosition1();
						Position op2 = opening.getPosition2();

						Point opos1 = chartdata.transPos(op1);
						Point opos2 = chartdata.transPos(op2);

						//check the openings....
						if( isIntersection(opos1, opos2, from, to) ){

							return opening;
						}
					} //for

				} //for

			}
		}
		return null;
	}


	//Determines if a wall, opening, or furnishing blocks the way
	public Opening nearOpening2(Point to, Point from){


		if( floor != null ){

			Vector rooms = floor.getRooms();

			for(int i=rooms.size()-1; i>=0; i-- ){

				Room rm = (Room)rooms.elementAt(i);
				Vector walls = rm.getWalls();

				for(int j=walls.size()-1; j>=0; j-- ){

					Wall wall = (Wall)walls.elementAt(j);

					Vector openings = wall.getOpenings();

					for(int k=openings.size()-1; k>=0; k-- ){

						Opening opening = (Opening)openings.elementAt(k);
						Position op1 = opening.getPosition1();
						Position op2 = opening.getPosition2();

						Point opos1 = chartdata.transPos(op1);
						Point opos2 = chartdata.transPos(op2);

						//check the openings....
						if( isIntersection(opos1, opos2, from, to) ){

							return opening;
						}
					} //for

				} //for

			}
		}
		return null;
	}


	//Determines if a wall, opening, or furnishing blocks the way
	public boolean canMove(Point to, Point from){

		boolean move = true;

		if( floor != null ){

			Vector rooms = floor.getRooms();

			for(int i=0; i<rooms.size(); i++ ){

				Room rm = (Room)rooms.elementAt(i);
				Vector walls = rm.getWalls();
				Vector furns = rm.getFurnishings();

				for(int j=0; j<walls.size(); j++ ){

					Wall wall = (Wall)walls.elementAt(j);
					Position p1 = wall.getPosition1();
					Position p2 = wall.getPosition2();

					Point pos1 = chartdata.transPos(p1);
					Point pos2 = chartdata.transPos(p2);

					//check the walls....
					if( isIntersection(pos1, pos2, from, to) )
						move = false;



					Vector openings = wall.getOpenings();

					for(int k=0; k<openings.size(); k++ ){

						Opening opening = (Opening)openings.elementAt(k);
						Position op1 = opening.getPosition1();
						Position op2 = opening.getPosition2();

						Point opos1 = chartdata.transPos(op1);
						Point opos2 = chartdata.transPos(op2);

						//check the openings....
						if( isIntersection(opos1, opos2, from, to) ){
							if( opening.isOpen() ){
								return true;
							}else{
								return false;
							}
						}
					} //for

				} //for

				for(int j=0; j<furns.size(); j++ ){

					Furnishing furn = (Furnishing)furns.elementAt(j);
					Position p1 = furn.getPosition1();
					Position p2 = furn.getPosition2();

					Point pos1 = chartdata.transPos(p1);
					Point pos2 = chartdata.transPos(p2);

					Point to1 = new Point(to.x+3, to.y+3);
					Point to2 = new Point(to.x+23, to.y+28);
					Point to3 = new Point(to.x+3, to.y+28);
					Point to4 = new Point(to.x+23, to.y+3);

					//check the furnishing...
					if( to1.x >= pos1.x && to1.x <= pos2.x && to1.y >= pos1.y && to1.y <= pos2.y ){
						return false;
					}else if( to1.x <= pos1.x && to1.x >= pos2.x && to1.y <= pos1.y && to1.y >= pos2.y ){
						return false;
					}else if( to1.x >= pos1.x && to1.x <= pos2.x && to1.y <= pos1.y && to1.y >= pos2.y ){
						return false;
					}else if( to1.x <= pos1.x && to1.x >= pos2.x && to1.y >= pos1.y && to1.y <= pos2.y ){
						return false;
					}
					if( to2.x >= pos1.x && to2.x <= pos2.x && to2.y >= pos1.y && to2.y <= pos2.y ){
						return false;
					}else if( to2.x <= pos1.x && to2.x >= pos2.x && to2.y <= pos1.y && to2.y >= pos2.y ){
						return false;
					}else if( to2.x >= pos1.x && to2.x <= pos2.x && to2.y <= pos1.y && to2.y >= pos2.y ){
						return false;
					}else if( to2.x <= pos1.x && to2.x >= pos2.x && to2.y >= pos1.y && to2.y <= pos2.y ){
						return false;
					}
					if( to3.x >= pos1.x && to3.x <= pos2.x && to3.y >= pos1.y && to3.y <= pos2.y ){
						return false;
					}else if( to3.x <= pos1.x && to3.x >= pos2.x && to3.y <= pos1.y && to3.y >= pos2.y ){
						return false;
					}else if( to3.x >= pos1.x && to3.x <= pos2.x && to3.y <= pos1.y && to3.y >= pos2.y ){
						return false;
					}else if( to3.x <= pos1.x && to3.x >= pos2.x && to3.y >= pos1.y && to3.y <= pos2.y ){
						return false;
					}
					if( to4.x >= pos1.x && to4.x <= pos2.x && to4.y >= pos1.y && to4.y <= pos2.y ){
						return false;
					}else if( to4.x <= pos1.x && to4.x >= pos2.x && to4.y <= pos1.y && to4.y >= pos2.y ){
						return false;
					}else if( to4.x >= pos1.x && to4.x <= pos2.x && to4.y <= pos1.y && to4.y >= pos2.y ){
						return false;
					}else if( to4.x <= pos1.x && to4.x >= pos2.x && to4.y >= pos1.y && to4.y <= pos2.y ){
						return false;
					}
				}

			} //rooms
		} //if

		return move;

	}


	public boolean isIntersection(Point pos1, Point pos2, Point from, Point to){

		boolean x = false;
		boolean y = false;
		to = new Point(to.x+3, to.y+3);
		from = new Point(from.x+3, from.y+3);
		Point to2 = new Point(to.x+23, to.y+28);

		double s = 0;
		double s1 = 0;
		double s2 = 0;

		try{
			s = (pos1.getY()-pos2.getY())/(pos1.getX()-pos2.getX());
		}catch(Exception ee){}
		try{
			s1 = (from.getY()-to.getY())/(from.getX()-to.getX());
		}catch(Exception ee){}
		try{
			s2 = (from.getY()-to2.getY())/(from.getX()-to2.getX());
		}catch(Exception ee){}

		if( s > 1000000 ) s = 10000;
		if( s < -1000000 ) s = -10000;
		if( s1 < -1000000 ) s1 = -10000;
		if( s2 < -1000000 ) s2 = -10000;
		if( s1 > 1000000 ) s1 = 10000;
		if( s2 > 1000000 ) s2 = 10000;
		if( new Double(s).isNaN() ) s = 0;
		if( new Double(s1).isNaN() ) s1 = 0;
		if( new Double(s2).isNaN() ) s2 = 0;

		//get the intsect point.
		double interx1 = 0;
		double intery1 = 0;
		double interx2 = 0;
		double intery2 = 0;
		try{
			interx1 = Math.round( (s*pos1.getX() - pos1.getY() - s1*from.getX() + from.getY())/(s-s1) );
		}catch(Exception ee){}
		try{
			interx2 = Math.round( (s*pos1.getX() - pos1.getY() - s2*from.getX() + from.getY())/(s-s2) );
		}catch(Exception ee){}

		try{
			if( (interx1-pos1.getX()) != 0 ){
				intery1 = Math.round( s*(interx1-pos1.getX())+pos1.getY() );
			}else{
				intery1 = Math.round( s1*(interx1-from.getX())+from.getY() );
			}
		}catch(Exception ee){}
		try{
			if( (interx2-pos1.getX()) != 0 ){
				intery2 = Math.round( s*(interx2-pos1.getX())+pos1.getY() );
			}else{
				intery2 = Math.round( s2*(interx2-from.getX())+from.getY() );
			}
		}catch(Exception ee){}

		//System.out.println(" Wall      "+pos1.x+", "+pos1.y +  "      "+pos2.x+",  "+pos2.y);
		//System.out.println(" Agent     "+from.x+", "+from.y +  "      "+to.x+",  "+to.y);
		//System.out.println(" Intersect "+interx1+", "+intery1);

		if( pos1.x >= interx1 && pos2.x <= interx1 ){

			if( from.x >= interx1 && to.x <= interx1 ){
				x = true;
			}else if( from.x <= interx1 && to.x >= interx1 ){
				x = true;
			}

		}else if( pos1.x <= interx1 && pos2.x >= interx1 ){
			if( from.x >= interx1 && to.x <= interx1 ){
				x = true;
			}else if( from.x <= interx1 && to.x >= interx1 ){
				x = true;
			}
		}

		if( pos1.y >= intery1 && pos2.y <= intery1 ){

			if( from.y >= intery1 && to.y <= intery1 ){
				y = true;
			}else if( from.y <= intery1 && to.y >= intery1 ){
				y = true;
			}

		}else if( pos1.y <= intery1 && pos2.y >= intery1 ){

			if( from.y >= intery1 && to.y <= intery1 ){
				y = true;
			}else if( from.y <= intery1 && to.y >= intery1 ){
				y = true;
			}
		}
		if( pos1.x >= interx2 && pos2.x <= interx2 ){

			if( from.x >= interx2 && to2.x <= interx2 ){
				x = true;
			}else if( from.x <= interx2 && to2.x >= interx2 ){
				x = true;
			}

		}else if( pos1.x <= interx2 && pos2.x >= interx2 ){
			if( from.x >= interx2 && to2.x <= interx2 ){
				x = true;
			}else if( from.x <= interx2 && to2.x >= interx2 ){
				x = true;
			}
		}

		if( pos1.y >= intery2 && pos2.y <= intery2 ){

			if( from.y >= intery2 && to2.y <= intery2 ){
				y = true;
			}else if( from.y <= intery2 && to2.y >= intery2 ){
				y = true;
			}

		}else if( pos1.y <= intery2 && pos2.y >= intery2 ){

			if( from.y >= intery2 && to2.y <= intery2 ){
				y = true;
			}else if( from.y <= intery2 && to2.y >= intery2 ){
				y = true;
			}
		}
		if( x && y )
			return true;
		else
			return false;
	}



	//If the given point is in the room then it must
	// have a wall on each side.
	public Room isRoom(Point p){

		if( floor != null ){

			Vector rooms = floor.getRooms();

			for(int i=0; i<rooms.size(); i++ ){

				Room rm = (Room)rooms.elementAt(i);
				Vector walls = rm.getWalls();

				boolean north = false;
				boolean south = false;
				boolean east = false;
				boolean west = false;

				for(int j=0; j<walls.size(); j++ ){

					Wall wall = (Wall)walls.elementAt(j);
					Position p1 = wall.getPosition1();
					Position p2 = wall.getPosition2();

					Point pos1 = chartdata.transPos(p1);
					Point pos2 = chartdata.transPos(p2);

					//north
					if( pos1.y > p.y && pos2.y > p.y ){
						if( pos1.x < p.x && pos2.x > p.x )
							north = true;
						if( pos1.x > p.x && pos2.x < p.x )
							north = true;
					//south
					}else if( pos1.y < p.y && pos2.y < p.y ){
						if( pos1.x < p.x && pos2.x > p.x )
							south = true;
						if( pos1.x > p.x && pos2.x < p.x )
							south = true;
					//west
					}else if( pos1.x > p.x && pos2.x > p.x ){
						if( pos1.y < p.y && pos2.y > p.y )
							east = true;
						if( pos1.y > p.y && pos2.y < p.y )
							east = true;
					//west
					}else if( pos1.x < p.x && pos2.x < p.x ){
						if( pos1.y < p.y && pos2.y > p.y )
							west = true;
						if( pos1.y > p.y && pos2.y < p.y )
							west = true;
					}
				}

				if( north && south && east && west ){
					return rm;
				}

			} //for



		}

		return null;
	}


	//If the given point is in the room then it must
	// have a wall on each side.
	public boolean isWall(Point p, Wall wall){

		boolean x = false;
		boolean y = false;


		Position p1 = wall.getPosition1();
		Position p2 = wall.getPosition2();

		Point pos1 = chartdata.transPos(p1);
		Point pos2 = chartdata.transPos(p2);

		//
		if( pos1.y >= p.y && pos2.y <= p.y ){
			x = true;
			//System.out.println("xxxxxxxxxxxxxxxxxxxxxxxxxx");
		//
		}else if( pos1.y <= p.y && pos2.y >= p.y ){
			x = true;
			//System.out.println("xxxxxxxxxxxxxxxx");
		//
		}
		if( pos1.x >= p.x && pos2.x <= p.x ){
			y = true;
			//System.out.println("yyyyyyyyyyyyyyyyyyyyyyyyyy");
		//
		}else if( pos1.x <= p.x && pos2.x >= p.x ){
			y = true;
			//System.out.println("yyyyyyyyyyyyyyyyyy");
		}


		if( x && y ){
			return true;
		}else{
			return false;
		}
	}

	//If the given point is in the room then it must
	// have a wall on each side.
	public boolean isOpening(Point p, Opening open){

		boolean x = false;
		boolean y = false;

		Position p1 = open.getPosition1();
		Position p2 = open.getPosition2();

		Point pos1 = chartdata.transPos(p1);
		Point pos2 = chartdata.transPos(p2);

		//
		if( pos1.y >= p.y && pos2.y <= p.y ){
			x = true;
		//
		}else if( pos1.y <= p.y && pos2.y >= p.y ){
			x = true;
		//
		}

		if( pos1.x >= p.x && pos2.x <= p.x ){
			y = true;
		//
		}else if( pos1.x <= p.x && pos2.x >= p.x ){
			y = true;
		}


		if( x && y ){
			return true;
		}else{
			return false;
		}
	}

    /* ===================================================================== */
    /** This method to tell this object whether or not it should paint the
     *  wireless range for each agent in the simulation.
     *
     *  @param  ranges   True if want to show wireless ranges.
     *
     */
	public boolean getRanges(){

		return ranges;

	} //method: setRanges

    /* ===================================================================== */
    /** This method to tell this object whether or not it should paint the
     *  wireless range for each agent in the simulation.
     *
     *  @param  ranges   True if want to show wireless ranges.
     *
     */
	public void setRanges(boolean ranges){

		this.ranges = ranges;

		// Show the wireless ranges now.
		repaint();

	} //method: setRanges


    /* ===================================================================== */
    /** This method returns the pixel distance between two points.
     *
     *  @param  p1   The first point.
     *  @param  p2   The second point.
     *
     */
	private double diff(Point p1, Point p2){

		double x = p1.getX() - p2.getX();
		if( x < 0 )
			x = x * -1;

		double y = p1.getY() - p2.getY();
		if( y < 0 )
			y = y * -1;


		return Math.sqrt( x*x + y*y );

	} //method: diff



    /* ===================================================================== */
    /** This method returns the pixel distance between two points.
     *
     *  @param  p1   The first point.
     *  @param  p2   The second point.
     *
     */
	private double getDegrees(Point p1, Point p2){

		double x = p1.getX() - p2.getX();
		double y = p1.getY() - p2.getY();


		double diff = Math.sqrt( x*x + y*y );

		return Math.toDegrees( Math.asin( (p1.getY()-p2.getY())/diff ) );

	} //method: diff


    /* ===================================================================== */
    /** This method returns the chartdata kept inside this object used for
     *  turning pixel coordinates into real GPS coordinates.
     *
     *  @return  ChartData   The chart data used in this object to return.
     *
     */
	public ChartData getChartData(){

		return chartdata;

	} //method: getChartData




    /* ===================================================================== */
    /** This method loads new chart data, by first reading the given xml file
     *  which contains map name and reference point information.  Then the new
     *  ChartData will give us the map image to use.
     *
     *  @param  name   file name for new data to use.
     *
     */
	public void loadChart(String name){

		this.setVisible(false);

		this.readData(name);

		if (img!=null) {img.flush();}

		img = getToolkit().getImage("files/maps/"+chartdata.imgFileName);


		this.setSize(this.getPreferredSize());


		this.setVisible(true);
		repaint();

	} //method: loadChart



    /* ===================================================================== */
    /** This method loads new chart data, by first reading the given xml file
     *  which contains map name and reference point information.
     *
     *  @param  file_name   Name of XML file that contains new ChartData info.
     *
     */
 	public void readData(String file_name){

    	chartdata = null;

		InputParser ip = new InputParser(file_name);

		try{
			chartdata=ChartData.loadChartData(ip);
		}catch (IOException e) {}


  	} //method: readData


} //class: ChartComp

