/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package state;

import state.events.*;
import agent.*;
import agent.autonomous.*;
import controller.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;
import java.io.*;
import java.util.*;
import java.awt.*;



/* ====================================================================== */
/** This class reads in a given XML file and parses it for a variety of
 *  values which it will store in its private variables.  It then provides
 *  methods for retrieving the values.
 *
 */
public class ScenarioParser{


    /** The directory where the configuration files are located. */
    public static final String DIRECTORY = "files/scenarios/";

    /** A default map. */
    private Vector users = new Vector();

    /** A default map. */
    private Vector points = new Vector();

	/** Our vector of reference points */
    private Vector events = new Vector();

	/** The XML doc we create from given filename. */
    private DocumentBuilder db = null;


	private Controller controller;

    /* ===================================================================== */
    /** This constructor builds up an XML Document of Nodes from the given XML
     *  file and then calls the method echo to retrieve the values of certain
     *  nodes.
     *
     *  @param  filename   The filename of the XML input file.
     *
     */
    public ScenarioParser(String filename, Controller controller) {

		this.controller = controller;

        boolean validation = false;
        boolean ignoreWhitespace = true;
        boolean ignoreComments = false;
        boolean putCDATAIntoText = false;
        boolean createEntityRefs = false;

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setValidating(validation);
        dbf.setIgnoringComments(ignoreComments);
        dbf.setIgnoringElementContentWhitespace(ignoreWhitespace);
        dbf.setCoalescing(putCDATAIntoText);
        dbf.setExpandEntityReferences(!createEntityRefs);
        try {
            db = dbf.newDocumentBuilder();
            Document doc = null;
            doc = db.parse(new File(DIRECTORY + filename));
            parse(doc);
        }catch(Exception e){
            System.out.println("Error openning input document: " +filename + " using system defaults");

        }

    } //Constructor



    /* ===================================================================== */
    /** This method takes in an Element and returns the concatenation of all
     *  of its text subnodes.
     *
     *  @param  e   The Element to analize
     *  @return the text from all the direct subnodes of node
     *
     */
    public String getText(Element e){
        String result = "";
        for (Node n = e.getFirstChild(); n != null; n = n.getNextSibling())
            try {
                result += ((CharacterData)n).getData();
            } catch (ClassCastException cce) {} //not a text node
        return result;
    }// getText




    /* ===================================================================== */
    /** This method takes in a Document node, which heads the XML tree
     *  structure.  The method will search through all the nodes for certain
     *  named nodes which contain values we are after.
     *
     *  @param  node   The root node for the XML Document.
     *
     */
    public void parse(Document doc){
        for (Node n = doc.getDocumentElement().getFirstChild(); n != null;  n = n.getNextSibling()) {
            try {

                Element e = (Element)n;
                String nodeName = e.getTagName();        //the node's name
                System.out.println("checking for the " + nodeName + " value");

              	if(nodeName.equals("people")) {

                    for (Node n1 = n.getFirstChild(); n1 != null;  n1 = n1.getNextSibling()) {
						Element e1 = (Element)n1;
						String nodeName1 = e1.getTagName();        //the node's name
                		System.out.println("   checking for the " + nodeName1 + " value");

                		if (nodeName1.equals("user")) {
							users.addElement(new User(e1.getAttribute("name"), getLocation(n1), Integer.parseInt(e1.getAttribute("port")), Integer.parseInt(e1.getAttribute("range")) ));

						}else if (nodeName1.equals("accesspoint")) {
							points.addElement(new AccessPoint(e1.getAttribute("name"), getLocation(n1), Integer.parseInt(e1.getAttribute("range")) ));

						}else if (nodeName1.equals("searcher")) {
							users.addElement(new Searcher(controller, getLocation(n1), Integer.parseInt(e1.getAttribute("port")),Integer.parseInt(e1.getAttribute("range")) ));

						}else if (nodeName1.equals("rescuer")) {
							users.addElement(new Rescuer(controller, getLocation(n1), Integer.parseInt(e1.getAttribute("port")),Integer.parseInt(e1.getAttribute("range")) ));
						}

					}

              	}else if(nodeName.equals("interaction")) {

                 	for (Node n1 = n.getFirstChild(); n1 != null;  n1 = n1.getNextSibling()) {
						Element e1 = (Element)n1;
						String nodeName1 = e1.getTagName();        //the node's name
                		System.out.println("   checking for the " + nodeName1 + " value");
                		if (nodeName1.equals("unievent")) {
							events.addElement(new UniEvent(Integer.parseInt(e1.getAttribute("to")), Integer.parseInt(e1.getAttribute("from")) ));
						}else if (nodeName1.equals("multievent")) {
							events.addElement(new MultiEvent( Integer.parseInt(e1.getAttribute("from")) ));
						}else if (nodeName1.equals("simevent")) {
							events.addElement(new SimEvent(Integer.parseInt(e1.getAttribute("to")), Integer.parseInt(e1.getAttribute("from")) ));
						}

					}
				} //if


            } catch (ClassCastException cce) {   //node was not an element
            } catch (NumberFormatException nfe) {  //numberic value wasn't there

            }//endtry

        }//endfor

    } //endmethod parse





	private Point getLocation(Node n){

		Element e = (Element)n;
        String nodeName = e.getTagName();        //the node's name

        if(nodeName.equals("user") || nodeName.equals("searcher") || nodeName.equals("rescuer") || nodeName.equals("accesspoint") ){

           	Node n1 = n.getFirstChild();
			Element e1 = (Element)n1;
			String nodeName1 = e1.getTagName();        //the node's name

            if (nodeName1.equals("location")) {

				return new Point(Integer.parseInt(e1.getAttribute("x")), Integer.parseInt(e1.getAttribute("y")) );
			}

        }
        return null;

	}


    /* ===================================================================== */
    /** This method is for getting the filename of the map image.
     *
     *  @return  String   The name of the map image.
     *
     */
    public Vector getUsers(){return users;}


    /* ===================================================================== */
    /** This method is for getting the filename of the map image.
     *
     *  @return  String   The name of the map image.
     *
     */
    public Vector getPoints(){return points;}


    /* ===================================================================== */
    /** This method is for getting the ref points of the map.
     *
     *  @return  Vector   A set of at least two reference points.
     *
     */
    public Vector getEvents(){return events;}



} //class: ScenarioParser
